<?php

namespace Drupal\dynamic_queues\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 * Class DynamicQueuesFilterForm.
 *
 * @package Drupal\dynamic_queues\Form
 */
class DynamicQueuesListsFilterForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'dynamic_queue_lists_filter_form',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'dynamic_queue_lists_filter_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $config = \Drupal::config('dynamic_queues.settings_form');
    $queue_type_lists = explode("|", $config->get('multiple_queue_names'));
    array_unshift($queue_type_lists, "Choose Queue Type");
    $queue_type_lists = array_combine($queue_type_lists, $queue_type_lists);
    $totalQueue = 0;

    if ($form_state->getValue('queue_types')) {
      $queue_types = $form_state->getValue('queue_types');
    }
    elseif (!empty(\Drupal::request()->query->get('queue_types'))) {
      // Use a default value.
      $queue_types = \Drupal::request()->query->get('queue_types');
      $queueName = \Drupal::request()->query->get('queue_lists_name');
      $query = \Drupal::database()->select('queue', 'q');
      $query->condition('name', "%" . \Drupal::database()->escapeLike($queueName) . "%", 'LIKE');
      $query->condition('expire', 0);
      $query->fields('q');
      $queue_item_exists = $query->execute()->fetchCol();

      $totalQueue = count($queue_item_exists);
    }
    else {
      $queue_types = '';
      $queueName = '';
    }

    $form['filters'] = [
      '#type'  => 'fieldset',
      '#title' => $this->t('Dynamic Queues Filter'),
      '#open'  => FALSE,
    ];
    $form['help'] = [
      '#type' => 'item',
      '#markup' => $this->t('Total Queue: @totalQueue', [
        '@totalQueue' => $totalQueue,
      ]),
    ];

    $form['filters']['queue_types'] = [
      '#type' => 'select',
      '#options' => $queue_type_lists,
      '#title' => 'Choose Queue Type',
      '#attributes' => [
        'class' => ['use-ajax'],
      ],
      '#ajax' => [
        'callback' => '::callbackQueueListTrigger',
        'wrapper' => 'dynamic_queue_lists',
        'event' => 'change',
        'method' => 'replace',
      ],
      '#default_value' => $queue_types,
    ];
    $form['filters']['queue_lists'] = [
      '#type' => 'select',
      '#title' => "Choose Queue list",
      '#attributes' => ['id' => ['selectQueue']],
      '#prefix' => '<div id="dynamic_queue_lists">',
      '#suffix' => '</div>',
      // When the form is rebuilt during ajax processing, the $selected_family
      // variable will now have the new value and so the options will change.
      '#options' => static::getQueueListsDetails($queue_types),
      '#default_value' => $queueName,
    ];
    $form['filters']['actions'] = [
      '#type'       => 'actions',
    ];
    $form['filters']['actions']['submit'] = [
      '#type'  => 'submit',
      '#value' => $this->t('Get Queues'),

    ];
    $form['filters']['actions']['reset'] = [
      '#type' => 'submit',
      '#value' => t('Reset'),
      '#attributes' => ['onclick' => "window.location.href='/admin/seia/dynamic-queue-lists'"],
    ];

    return $form;

  }

  /**
   * Callback function.
   */
  public function callbackQueueListTrigger(array $form, FormStateInterface $form_state) {
    return $form['filters']['queue_lists'];
  }

  /**
   * Fetch queue lists details.
   */
  public static function getQueueListsDetails($queue_type = '') {
    if (!empty($queue_type)) {
      $queue_name = 'dynamic_queues:' . $queue_type . '_queue_';
      $query = \Drupal::database()->select('queue', 'q');
      $query->condition('name', "%" . \Drupal::database()->escapeLike($queue_name) . "%", 'LIKE');
      // $query->condition('expire', 0);
      $query->fields('q', ['name']);
      $queue_lists = $query->distinct()->execute()->fetchCol();

      if (count($queue_lists) == 0) {
        $queue_lists = ['none' => 'None'];
      }
      else {
        array_unshift($queue_lists, "Choose Queue");
      }
      $queue_lists = array_combine($queue_lists, $queue_lists);

    }
    else {
      $queue_lists = ['none' => 'None'];
    }
    return $queue_lists;

  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $field = $form_state->getValues();
    $queue_lists_name = $field["queue_lists"];
    $queue_types = $field["queue_types"];
    $url = Url::fromRoute('dynamic_queues.queue_lists')
      ->setRouteParameters(['queue_types' => $queue_types, 'queue_lists_name' => $queue_lists_name]);
    $form_state->setRedirectUrl($url);
  }

}
